@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -29,6 +28,7 @@
 #include <ocserv-args.h>
 #include <autoopts/options.h>
 #include <limits.h>
+#include <common.h>
 #include <c-strcase.h>
 
 #include <vpn.h>
@@ -39,10 +39,6 @@
 static const char* pid_file = NULL;
 static const char* cfg_file = DEFAULT_CFG_FILE;
 
-#define MAX_ENTRIES 64
-
-enum option_types { OPTION_NUMERIC, OPTION_STRING, OPTION_BOOLEAN, OPTION_MULTI_LINE };
-
 struct cfg_options {
 	const char* name;
 	unsigned type;
@@ -83,6 +79,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "tls-priorities", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "chroot-dir", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "mtu", .type = OPTION_NUMERIC, .mandatory = 0 },
+	{ .name = "net-priority", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "output-buffer", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "cookie-validity", .type = OPTION_NUMERIC, .mandatory = 1 },
 	{ .name = "auth-timeout", .type = OPTION_NUMERIC, .mandatory = 0 },
@@ -90,9 +87,13 @@ static struct cfg_options available_options[] = {
 	{ .name = "min-reauth-time", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "max-same-clients", .type = OPTION_NUMERIC, .mandatory = 0 },
 
+	{ .name = "rx-data-per-sec", .type = OPTION_NUMERIC, .mandatory = 0 },
+	{ .name = "tx-data-per-sec", .type = OPTION_NUMERIC, .mandatory = 0 },
+
 	{ .name = "run-as-user", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "run-as-group", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "device", .type = OPTION_STRING, .mandatory = 1 },
+	{ .name = "cgroup", .type = OPTION_STRING, .mandatory = 0 },
 
 	{ .name = "ipv4-network", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "ipv4-netmask", .type = OPTION_STRING, .mandatory = 0 },
@@ -100,8 +101,13 @@ static struct cfg_options available_options[] = {
 	{ .name = "ipv4-nbns", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "ipv6-network", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "ipv6-netmask", .type = OPTION_STRING, .mandatory = 0 },
+	{ .name = "ipv6-prefix", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "ipv6-dns", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "ipv6-nbns", .type = OPTION_STRING, .mandatory = 0 },
+	{ .name = "route-add-cmd", .type = OPTION_STRING, .mandatory = 0 },
+	{ .name = "route-del-cmd", .type = OPTION_STRING, .mandatory = 0 },
+	{ .name = "config-per-user", .type = OPTION_STRING, .mandatory = 0 },
+	{ .name = "config-per-group", .type = OPTION_STRING, .mandatory = 0 },
 };
 
 static const tOptionValue* get_option(const char* name, unsigned * mand)
@@ -124,13 +130,13 @@ unsigned j;
 	if (val != NULL && val->valType == OPARG_TYPE_STRING) { \
 		if (s_name == NULL) { \
 			num = 0; \
-			s_name = malloc(sizeof(char*)*MAX_ENTRIES); \
+			s_name = malloc(sizeof(char*)*MAX_CONFIG_ENTRIES); \
 			do { \
 			        if (val && !strcmp(val->pzName, name)==0) \
 					continue; \
 			        s_name[num] = strdup(val->v.strVal); \
 			        num++; \
-			        if (num>=MAX_ENTRIES) \
+			        if (num>=MAX_CONFIG_ENTRIES) \
 			        break; \
 		      } while((val = optionNextValue(pov, val)) != NULL); \
 		      s_name[num] = NULL; \
@@ -173,6 +179,22 @@ unsigned j;
 		exit(1); \
 	}
 
+#define READ_PRIO_TOS(name, s_name) \
+	val = get_option(name, &mand); \
+	if (val != NULL) { \
+		if (val->valType == OPARG_TYPE_STRING) { \
+			if (strncmp(val->v.strVal, "0x", 2) == 0) { \
+				s_name = strtol(val->v.strVal, NULL, 16); \
+				s_name = TOS_PACK(s_name); \
+			} else { \
+				s_name = atoi(val->v.strVal); \
+				s_name++; \
+			} \
+		} \
+	} else if (mand != 0) { \
+		fprintf(stderr, "Configuration option %s is mandatory.\n", name); \
+		exit(1); \
+	}
 
 
 static int handle_option(const tOptionValue* val)
@@ -206,6 +228,7 @@ const tOptionValue* val, *prev;
 unsigned j, mand;
 char** auth = NULL;
 unsigned auth_size = 0;
+unsigned prefix = 0;
 
 	pov = configFileLoad(file);
 	if (pov == NULL) {
@@ -312,8 +335,16 @@ unsigned auth_size = 0;
 	READ_STRING("chroot-dir", config->chroot_dir);
 
 	READ_NUMERIC("mtu", config->default_mtu);
+
+	READ_PRIO_TOS("net-priority", config->net_priority);
+
 	READ_NUMERIC("output-buffer", config->output_buffer);
 
+	READ_NUMERIC("rx-data-per-sec", config->rx_per_sec);
+	READ_NUMERIC("tx-data-per-sec", config->tx_per_sec);
+	config->rx_per_sec /= 1000; /* in kb */
+	config->tx_per_sec /= 1000;
+
 	READ_NUMERIC("cookie-validity", config->cookie_validity);
 	READ_NUMERIC("auth-timeout", config->auth_timeout);
 	READ_NUMERIC("max-clients", config->max_clients);
@@ -341,6 +372,7 @@ unsigned auth_size = 0;
 	}
 
 	READ_STRING("device", config->network.name);
+	READ_STRING("cgroup", config->cgroup);
 
 	READ_STRING("ipv4-network", config->network.ipv4);
 	READ_STRING("ipv4-netmask", config->network.ipv4_netmask);
@@ -348,12 +380,22 @@ unsigned auth_size = 0;
 
 	READ_STRING("ipv6-network", config->network.ipv6);
 	READ_STRING("ipv6-netmask", config->network.ipv6_netmask);
+
+	READ_NUMERIC("ipv6-prefix", prefix);
+	if (prefix > 0) 
+		config->network.ipv6_netmask = ipv6_prefix_to_mask(prefix);
+	
 	READ_STRING("ipv6-dns", config->network.ipv6_dns);
 
 	READ_STRING("ipv4-nbns", config->network.ipv4_nbns);
 	READ_STRING("ipv6-nbns", config->network.ipv6_nbns);
 
 	READ_MULTI_LINE("route", config->network.routes, config->network.routes_size);
+
+	READ_STRING("route-add-cmd", config->route_add_cmd);
+	READ_STRING("route-del-cmd", config->route_del_cmd);
+	READ_STRING("config-per-user", config->per_user_dir);
+	READ_STRING("config-per-group", config->per_group_dir);
 		
 	optionUnloadNested(pov);
 }
@@ -432,9 +474,6 @@ static void check_cfg( struct cfg_st *config)
 	if (config->dpd == 0)
 		config->keepalive = 60;
 
-	if (config->udp_port == 0)
-		config->udp_port = config->port;
-	
 	if (config->priorities == NULL)
 		config->priorities = "NORMAL:%SERVER_PRECEDENCE:%COMPAT";
 }
@@ -449,9 +488,15 @@ int cmd_parser (int argc, char **argv, struct cfg_st* config)
 	if (HAVE_OPT(FOREGROUND))
 		config->foreground = 1;
 
+	if (HAVE_OPT(PID_FILE))
+		pid_file = OPT_ARG(PID_FILE);
+
 	if (HAVE_OPT(TLS_DEBUG))
 		config->tls_debug = 1;
 
+	if (HAVE_OPT(HTTP_DEBUG))
+		config->http_debug = 1;
+
 	if (HAVE_OPT(DEBUG))
 		config->debug = 1;
 	
@@ -480,6 +525,11 @@ unsigned i;
 	DEL(config->xml_config_hash);
 	DEL(config->cert_hash);
 #endif
+	DEL(config->cgroup);
+	DEL(config->route_add_cmd);
+	DEL(config->route_del_cmd);
+	DEL(config->per_user_dir);
+	DEL(config->per_group_dir);
 	DEL(config->socket_file_prefix);
 	DEL(config->default_domain);
 	DEL(config->plain_passwd);
@@ -514,6 +564,7 @@ unsigned i;
 		DEL(config->cert[i]);
 	DEL(config->cert);
 	DEL(config->network.routes);
+
 	memset(config, 0, sizeof(*config));
 
 	parse_cfg_file(cfg_file, config);
