@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,6 +31,7 @@
 #include <limits.h>
 #include <sys/stat.h>
 
+#include <ip-lease.h>
 #include <main.h>
 #include <cookies.h>
 
@@ -123,3 +123,31 @@ cleanup:
 	return ret;
 
 }
+
+int generate_cookie(main_server_st *s, struct proc_st* proc)
+{
+int ret;
+struct stored_cookie_st sc;
+
+        ret = gnutls_rnd(GNUTLS_RND_NONCE, proc->session_id, sizeof(proc->session_id));
+        if (ret < 0)
+                return -1;
+        
+        proc->session_id_size = sizeof(proc->session_id);
+
+	memcpy(sc.username, proc->username, sizeof(proc->username));
+	memcpy(sc.groupname, proc->groupname, sizeof(proc->groupname));
+	memcpy(sc.hostname, proc->hostname, sizeof(proc->hostname));
+	memcpy(sc.session_id, proc->session_id, sizeof(proc->session_id));
+	
+	sc.expiration = time(0) + s->config->cookie_validity;
+	
+	memcpy(sc.ipv4_seed, SA_IN_U8_P(&proc->ipv4->lip), 4);
+
+	ret = encrypt_cookie(s, &sc, proc->cookie, sizeof(proc->cookie));
+	if (ret < 0)
+		return -1;
+
+	return 0;
+}
+
