@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -75,6 +74,13 @@ void __attribute__ ((format(printf, 3, 4)))
 	if (priority == LOG_DEBUG && ws->config->debug == 0)
 		return;
 
+	if (priority == LOG_HTTP_DEBUG) {
+	    if (ws->config->http_debug == 0)
+                return;
+            else
+                priority = LOG_INFO;
+        }
+
 	ip = human_addr((void*)&ws->remote_addr, ws->remote_addr_len,
 			    ipbuf, sizeof(ipbuf));
 
@@ -104,6 +110,13 @@ void __attribute__ ((format(printf, 4, 5)))
 	if (priority == LOG_DEBUG && s->config->debug == 0)
 		return;
 
+	if (priority == LOG_HTTP_DEBUG) {
+	    if (s->config->http_debug == 0)
+                return;
+            else
+                priority = LOG_DEBUG;
+        }
+
 	if (proc) {
 		ip = human_addr((void*)&proc->remote_addr, proc->remote_addr_len,
 			    ipbuf, sizeof(ipbuf));
