@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -37,8 +36,11 @@
 #include <sys/un.h>
 #include <cloexec.h>
 #include "ipc.h"
+#include "str.h"
 #include "setproctitle.h"
 #include <sec-mod.h>
+#include <route-add.h>
+#include <ip-lease.h>
 
 #include <vpn.h>
 #include <cookies.h>
@@ -53,10 +55,10 @@ int fd, ret, e;
 struct ifreq ifr;
 const char* name;
 
-	if (proc->lease == NULL)
+	if (proc->tun_lease.name[0] == 0)
 		return -1;
 
-	name = proc->lease->name;
+	name = proc->tun_lease.name;
 
 	mslog(s, proc, LOG_DEBUG, "setting %s MTU to %u", name, mtu);
 	fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -83,7 +85,7 @@ fail:
 
 int send_udp_fd(main_server_st* s, struct proc_st * proc, int fd)
 {
-	struct iovec iov[2];
+	struct iovec iov[1];
 	uint8_t cmd = CMD_UDP_FD;
 	struct msghdr hdr;
 	union {
@@ -91,13 +93,13 @@ int send_udp_fd(main_server_st* s, struct proc_st * proc, int fd)
 		char control[CMSG_SPACE(sizeof(int))];
 	} control_un;
 	struct cmsghdr  *cmptr;	
-
+	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
 	iov[0].iov_base = &cmd;
 	iov[0].iov_len = 1;
-	hdr.msg_iovlen++;
 
+	hdr.msg_iovlen = 1;
 	hdr.msg_iov = iov;
 
 	hdr.msg_control = control_un.control;
@@ -109,7 +111,12 @@ int send_udp_fd(main_server_st* s, struct proc_st * proc, int fd)
 	cmptr->cmsg_type = SCM_RIGHTS;
 	memcpy(CMSG_DATA(cmptr), &fd, sizeof(int));
 	
-	return(sendmsg(proc->fd, &hdr, 0));
+	ret = sendmsg(proc->fd, &hdr, 0);
+	if (ret < 0) {
+		int e = errno;
+		mslog(s, proc, LOG_ERR, "sendmsg: %s", strerror(e));
+	}
+	return ret;
 }
 
 int handle_script_exit(main_server_st *s, struct proc_st* proc, int code)
@@ -117,12 +124,16 @@ int handle_script_exit(main_server_st *s, struct proc_st* proc, int code)
 int ret;
 
 	if (code == 0) {
+		proc->auth_status = PS_AUTH_COMPLETED;
+
 		ret = send_auth_reply(s, proc, REP_AUTH_OK);
 		if (ret < 0) {
-			mslog(s, proc, LOG_ERR, "could not send reply auth cmd.");
+			mslog(s, proc, LOG_ERR, "could not send auth reply cmd.");
 			ret = ERR_BAD_COMMAND;
 			goto fail;
 		}
+		
+		apply_iroutes(s, proc);
 	} else {
 		mslog(s, proc, LOG_INFO, "failed authentication attempt for user '%s'", proc->username);
 		ret = send_auth_reply( s, proc, REP_AUTH_FAILED);
@@ -138,15 +149,183 @@ fail:
 	/* we close the lease tun fd both on success and failure.
 	 * The parent doesn't need to keep the tunfd.
 	 */
-	if (proc->lease) {
-		if (proc->lease->fd >= 0)
-			close(proc->lease->fd);
-		proc->lease->fd = -1;
+	if (proc->tun_lease.name[0] != 0) {
+		if (proc->tun_lease.fd >= 0)
+			close(proc->tun_lease.fd);
+		proc->tun_lease.fd = -1;
 	}
 
 	return ret;
 }
 
+static int read_additional_config_file(main_server_st* s, struct proc_st* proc, const char* file, const char* type)
+{
+struct group_cfg_st cfg;
+int ret;
+unsigned i;
+
+	if (access(file, R_OK) == 0) {
+		mslog(s, proc, LOG_DEBUG, "Loading %s configuration '%s'", type, file);
+
+		ret = parse_group_cfg_file(s, file, &cfg);
+		if (ret < 0)
+			return ERR_READ_CONFIG;
+		
+		if (cfg.routes_size > 0) {
+			if (proc->config.routes == NULL) {
+				proc->config.routes = cfg.routes;
+				proc->config.routes_size = cfg.routes_size;
+				
+				cfg.routes = NULL;
+				cfg.routes_size = 0;
+			} else {
+				proc->config.routes = safe_realloc(proc->config.routes, (proc->config.routes_size + cfg.routes_size) * sizeof(proc->config.routes[0]));
+				if (proc->config.routes == NULL)
+					return ERR_MEM;
+				
+				for (i=0;i<cfg.routes_size;i++) {
+					proc->config.routes[proc->config.routes_size] = cfg.routes[i];
+					cfg.routes[i] = NULL;
+					proc->config.routes_size++;
+				}
+			}
+		}
+
+		if (proc->config.iroutes == NULL) {
+			proc->config.iroutes = cfg.iroutes;
+			proc->config.iroutes_size = cfg.iroutes_size;
+				
+			cfg.iroutes = NULL;
+			cfg.iroutes_size = 0;
+		}
+
+		if (proc->config.ipv4_dns == NULL) {
+			proc->config.ipv4_dns = cfg.ipv4_dns;
+			cfg.ipv4_dns = NULL;
+		}
+
+		if (proc->config.ipv6_dns == NULL) {
+			proc->config.ipv6_dns = cfg.ipv6_dns;
+			cfg.ipv6_dns = NULL;
+		}
+
+		if (proc->config.ipv4_nbns == NULL) {
+			proc->config.ipv4_nbns = cfg.ipv4_nbns;
+			cfg.ipv4_nbns = NULL;
+		}
+
+		if (proc->config.ipv6_nbns == NULL) {
+			proc->config.ipv6_nbns = cfg.ipv6_nbns;
+			cfg.ipv6_nbns = NULL;
+		}
+
+		if (proc->config.ipv4_network == NULL) {
+			proc->config.ipv4_network = cfg.ipv4_network;
+			cfg.ipv4_network = NULL;
+		}
+
+		if (proc->config.ipv6_network == NULL) {
+			proc->config.ipv6_network = cfg.ipv6_network;
+			cfg.ipv6_network = NULL;
+		}
+
+		if (proc->config.ipv4_netmask == NULL) {
+			proc->config.ipv4_netmask = cfg.ipv4_netmask;
+			cfg.ipv4_netmask = NULL;
+		}
+
+		if (proc->config.ipv6_netmask == NULL) {
+			proc->config.ipv6_netmask = cfg.ipv6_netmask;
+			cfg.ipv6_netmask = NULL;
+		}
+
+		if (proc->config.cgroup == NULL) {
+			proc->config.cgroup = cfg.cgroup;
+			cfg.cgroup = NULL;
+		}
+
+		if (proc->config.rx_per_sec == 0) {
+			proc->config.rx_per_sec = cfg.rx_per_sec;
+		}
+
+		if (proc->config.tx_per_sec == 0) {
+			proc->config.tx_per_sec = cfg.tx_per_sec;
+		}
+
+		if (proc->config.net_priority == 0) {
+			proc->config.net_priority = cfg.net_priority;
+		}
+
+		del_additional_config(&cfg);
+
+	} else
+		mslog(s, proc, LOG_DEBUG, "No %s configuration for '%s'", type, proc->username);
+	
+	return 0;
+}
+
+
+static int read_additional_config(struct main_server_st* s, struct proc_st* proc)
+{
+char file[_POSIX_PATH_MAX];
+int ret;
+
+	memset(&proc->config, 0, sizeof(proc->config));
+
+	if (s->config->per_user_dir != NULL) {
+		snprintf(file, sizeof(file), "%s/%s", s->config->per_user_dir, proc->username);
+
+		ret = read_additional_config_file(s, proc, file, "user");
+		if (ret < 0)
+			return ret;
+	}
+
+	if (s->config->per_group_dir != NULL && proc->groupname[0] != 0) {
+		snprintf(file, sizeof(file), "%s/%s", s->config->per_group_dir, proc->groupname);
+
+		ret = read_additional_config_file(s, proc, file, "group");
+		if (ret < 0)
+			return ret;
+	}
+	
+	if (proc->config.cgroup != NULL) {
+		put_into_cgroup(s, proc->config.cgroup, proc->pid);
+	}
+
+	return 0;
+}
+
+/* k: whether to kill the process
+ */
+void remove_proc(main_server_st* s, struct proc_st *proc, unsigned k)
+{
+	if (k)
+		kill(proc->pid, SIGTERM);
+
+	user_disconnected(s, proc);
+
+	/* close the intercomm fd */
+	if (proc->fd >= 0)
+		close(proc->fd);
+	proc->fd = -1;
+	proc->pid = -1;
+	
+	remove_iroutes(s, proc);
+	del_additional_config(&proc->config);
+	
+	if (proc->auth_ctx != NULL)
+		proc_auth_deinit(s, proc);
+
+	if (proc->ipv4 || proc->ipv6)
+		remove_ip_leases(s, proc);
+
+	list_del(&proc->list);
+	free(proc);
+	s->active_clients--;
+}
+
+
+/* This is the function after which proc is populated */
 static int accept_user(main_server_st *s, struct proc_st* proc, unsigned cmd)
 {
 int ret;
@@ -162,6 +341,12 @@ const char* group;
 		return ret;
 	}
 
+	ret = read_additional_config(s, proc);
+	if (ret < 0) {
+		mslog(s, proc, LOG_ERR, "error reading additional configuration");
+		return ERR_READ_CONFIG;
+	}
+
 	ret = open_tun(s, proc);
 	if (ret < 0) {
 		return -1;
@@ -185,15 +370,11 @@ const char* group;
 
 	/* do scripts and utmp */
 	ret = user_connected(s, proc);
-	if (ret == ERR_WAIT_FOR_SCRIPT) {
-		return 0;
-	}
-
 	if (ret < 0) {
 		mslog(s, proc, LOG_INFO, "user '%s' disconnected due to script", proc->username);
 	}
 	
-	return 0;
+	return ret;
 }
 
 int handle_commands(main_server_st *s, struct proc_st* proc)
@@ -232,7 +413,9 @@ int handle_commands(main_server_st *s, struct proc_st* proc)
 	}
 
 	if (ret == 0) {
-		return -1;
+		e = errno;
+		mslog(s, proc, LOG_ERR, "command socket closed");
+		return ERR_WORKER_TERMINATED;
 	}
 
 	cmd_data_len = ret - 1;
@@ -312,6 +495,13 @@ int handle_commands(main_server_st *s, struct proc_st* proc)
 					return ret;
 				}
 				break; /* wait for another command */
+			} else if (ret == 0) {
+				ret = accept_user(s, proc, cmd);
+				if (ret < 0) {
+					goto cleanup;
+				}
+				proc->auth_status = PS_AUTH_COMPLETED;
+				goto cleanup;
 			} else if (ret < 0) {
 				add_to_ip_ban_list(s, &proc->remote_addr, proc->remote_addr_len);
 				goto cleanup;
@@ -383,8 +573,12 @@ int handle_commands(main_server_st *s, struct proc_st* proc)
 			proc->auth_status = PS_AUTH_COMPLETED;
 
 cleanup:
-			/* no script was called. Handle it as a successful script call. */
-			return handle_script_exit(s, proc, ret);
+			if (ret == ERR_WAIT_FOR_SCRIPT)
+				return 0;
+			else {
+				/* no script was called. Handle it as a successful script call. */
+				return handle_script_exit(s, proc, ret);
+			}
 
 		default:
 			mslog(s, proc, LOG_ERR, "unknown CMD 0x%x.", (unsigned)cmd);
@@ -424,7 +618,7 @@ struct banned_st *btmp, *bpos;
 void expire_banned(main_server_st* s)
 {
 time_t now = time(0);
-struct banned_st *btmp, *bpos;
+struct banned_st *btmp = NULL, *bpos;
 
 	if (s->config->min_reauth_time == 0)
 		return;
@@ -490,3 +684,51 @@ const char *p;
 		exit(1);
 	}
 }
+
+/* Puts the provided PIN into the config's cgroup */
+void put_into_cgroup(main_server_st * s, const char* _cgroup, pid_t pid)
+{
+char* name, *p, *savep;
+char cgroup[128];
+char file[_POSIX_PATH_MAX];
+FILE* fd;
+
+	if (_cgroup == NULL)
+		return;
+
+#ifdef __linux__
+	 /* format: cpu,memory:cgroup-name */
+	 snprintf(cgroup, sizeof(cgroup), "%s", _cgroup);
+	 
+	 name = strchr(cgroup, ':');
+	 if (name == NULL) {
+		mslog(s, NULL, LOG_ERR, "error parsing cgroup name: %s", cgroup);
+		return;
+	 }
+	 name[0] = 0;
+	 name++;
+	 
+	 p = strtok_r(cgroup, ",", &savep);
+	 while (p != NULL) {
+		mslog(s, NULL, LOG_DEBUG, "putting process %u to cgroup '%s:%s'", (unsigned)pid, p, name);
+
+	 	snprintf(file, sizeof(file), "/sys/fs/cgroup/%s/%s/tasks", p, name);
+	
+	 	fd = fopen(file, "w");
+	 	if (fd == NULL) {
+			mslog(s, NULL, LOG_ERR, "cannot open: %s", file);
+			return;
+		}
+
+		if (fprintf(fd, "%u", (unsigned)pid) <= 0) {
+			mslog(s, NULL, LOG_ERR, "could not write to: %s", file);
+		}
+		fclose(fd);
+		p = strtok_r(NULL, ",", &savep);
+	 }
+	 
+	 return;
+#else
+	mslog(s, NULL, LOG_DEBUG, "Ignoring cgroup option as it is not supported on this system");
+#endif
+}
