@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -45,10 +44,13 @@
 #endif
 
 #include <main.h>
+#include <route-add.h>
 #include <main-auth.h>
 #include <worker.h>
 #include <cookies.h>
 #include <tun.h>
+#include <grp.h>
+#include <ip-lease.h>
 #include <ccan/list/list.h>
 
 int syslog_open = 0;
@@ -207,29 +209,31 @@ listen_ports(struct cfg_st* config, struct listen_list_st *list, const char *nod
 		exit(1);
 	}
 
-	snprintf(portname, sizeof(portname), "%d", config->udp_port);
+	if (config->udp_port) {
+		snprintf(portname, sizeof(portname), "%d", config->udp_port);
 
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_socktype = SOCK_DGRAM;
-	hints.ai_flags = AI_PASSIVE
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_flags = AI_PASSIVE
 #ifdef AI_ADDRCONFIG
-	    | AI_ADDRCONFIG
+		    | AI_ADDRCONFIG
 #endif
-	    ;
+		    ;
 
-	ret = getaddrinfo(node, portname, &hints, &res);
-	if (ret != 0) {
-		fprintf(stderr, "getaddrinfo() failed: %s\n",
-			gai_strerror(ret));
-		return -1;
-	}
+		ret = getaddrinfo(node, portname, &hints, &res);
+		if (ret != 0) {
+			fprintf(stderr, "getaddrinfo() failed: %s\n",
+				gai_strerror(ret));
+			return -1;
+		}
 
-	ret = _listen_ports(config, res, list);
-	if (ret < 0) {
-		return -1;
-	}
+		ret = _listen_ports(config, res, list);
+		if (ret < 0) {
+			return -1;
+		}
 	
-	freeaddrinfo(res);
+		freeaddrinfo(res);
+	}
 
 	return 0;
 }
@@ -289,34 +293,12 @@ int s, y, e;
 	return 0;
 }
 
-static void remove_proc(main_server_st* s, struct proc_st *proc, unsigned k)
-{
-	if (k)
-		kill(proc->pid, SIGTERM);
-
-	user_disconnected(s, proc);
-
-	/* close the intercomm fd */
-	if (proc->fd >= 0)
-		close(proc->fd);
-	proc->fd = -1;
-	proc->pid = -1;
-	
-	if (proc->auth_ctx != NULL)
-		proc_auth_deinit(s, proc);
-
-	if (proc->lease)
-		proc->lease->in_use = 0;
-	list_del(&proc->list);
-	free(proc);
-	s->active_clients--;
-}
 
 static void cleanup_children(main_server_st *s)
 {
 int status, estatus, ret;
 pid_t pid;
-struct script_wait_st *stmp, *spos;
+struct script_wait_st *stmp = NULL, *spos;
 
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 		estatus = WEXITSTATUS(status);
@@ -332,7 +314,7 @@ struct script_wait_st *stmp, *spos;
 				mslog(s, stmp->proc, LOG_DEBUG, "%s-script exit status: %u", stmp->up?"connect":"disconnect", estatus);
 				ret = handle_script_exit(s, stmp->proc, estatus);
 				if (ret < 0)
-					remove_proc(s, stmp->proc, 0);
+					remove_proc(s, stmp->proc, 1);
 				list_del(&stmp->list);
 				free(stmp);
 				break;
@@ -367,7 +349,13 @@ static void drop_privileges(main_server_st* s)
 	struct rlimit rl;
 
 	if (s->config->chroot_dir) {
-		chdir(s->config->chroot_dir);
+		ret = chdir(s->config->chroot_dir);
+		if (ret != 0) {
+			e = errno;
+			mslog(s, NULL, LOG_ERR, "cannot chdir to %s: %s", s->config->chroot_dir, strerror(e));
+			exit(1);
+		}
+
 		ret = chroot(s->config->chroot_dir);
 		if (ret != 0) {
 			e = errno;
@@ -383,7 +371,14 @@ static void drop_privileges(main_server_st* s)
 			mslog(s, NULL, LOG_ERR, "cannot set gid to %d: %s\n",
 			       (int) s->config->gid, strerror(e));
 			exit(1);
-
+		}
+		
+		ret = setgroups(1, &s->config->gid);
+		if (ret < 0) {
+			e = errno;
+			mslog(s, NULL, LOG_ERR, "cannot set groups to %d: %s\n",
+			       (int) s->config->gid, strerror(e));
+			exit(1);
 		}
 	}
 
@@ -408,7 +403,16 @@ static void drop_privileges(main_server_st* s)
 	}
 
 #if 0
-#define MAX_WORKER_MEM (4*1024*1024)
+	rl.rlim_cur = 0;
+	rl.rlim_max = 0;
+	ret = setrlimit(RLIMIT_FSIZE, &rl);
+	if (ret < 0) {
+		e = errno;
+		mslog(s, NULL, LOG_ERR, "cannot enforce FSIZE limit: %s\n",
+		       strerror(e));
+	}
+
+#define MAX_WORKER_MEM (16*1024*1024)
 	if (s->config->debug == 0) {
 		rl.rlim_cur = MAX_WORKER_MEM;
 		rl.rlim_max = MAX_WORKER_MEM;
@@ -427,10 +431,10 @@ static void drop_privileges(main_server_st* s)
  */
 void clear_lists(main_server_st *s)
 {
-	struct listener_st *ltmp, *lpos;
-	struct proc_st *ctmp, *cpos;
-	struct script_wait_st *script_tmp, *script_pos;
-	struct banned_st *btmp, *bpos;
+	struct listener_st *ltmp = NULL, *lpos;
+	struct proc_st *ctmp = NULL, *cpos;
+	struct script_wait_st *script_tmp = NULL, *script_pos;
+	struct banned_st *btmp = NULL, *bpos;
 
 	list_for_each_safe(&s->llist.head, ltmp, lpos, list) {
 		close(ltmp->fd);
@@ -459,11 +463,14 @@ void clear_lists(main_server_st *s)
 	}
 
 	tls_cache_deinit(s->tls_db);
+	
+	ip_lease_deinit(&s->ip_leases);
+
 }
 
 static void kill_children(main_server_st* s)
 {
-	struct proc_st *ctmp;
+	struct proc_st *ctmp = NULL;
 
 	/* kill the security module server */
 	kill(s->sec_mod_pid, SIGTERM);
@@ -486,6 +493,10 @@ static void handle_reload(int signo)
 	reload_conf = 1;
 }
 
+/* A UDP fd will not be forwarded to worker process before this number of
+ * seconds has passed. That is to prevent a duplicate message messing the worker.
+ */
+#define UDP_FD_RESEND_TIME 60
 
 #define RECORD_PAYLOAD_POS 13
 #define HANDSHAKE_SESSION_ID_POS 46
@@ -493,13 +504,14 @@ static int forward_udp_to_owner(main_server_st* s, struct listener_st *listener)
 {
 int ret, e;
 struct sockaddr_storage cli_addr;
-struct proc_st *ctmp;
+struct proc_st *ctmp = NULL;
 socklen_t cli_addr_size;
 uint8_t buffer[1024];
 uint8_t  *session_id;
 int session_id_size;
 ssize_t buffer_size;
 int connected = 0;
+time_t now;
 
 	/* first receive from the correct client and connect socket */
 	cli_addr_size = sizeof(cli_addr);
@@ -533,10 +545,12 @@ int connected = 0;
 	session_id = &buffer[RECORD_PAYLOAD_POS+HANDSHAKE_SESSION_ID_POS+1];
 
 	/* search for the IP and the session ID in all procs */
+	now = time(0);
 
 	list_for_each(&s->clist.head, ctmp, list) {
 		if (session_id_size == ctmp->session_id_size &&
-			memcmp(session_id, ctmp->session_id, session_id_size) == 0) {
+			memcmp(session_id, ctmp->session_id, session_id_size) == 0 &&
+			(now - ctmp->udp_fd_receive_time > UDP_FD_RESEND_TIME)) {
 
 			ret = connect(listener->fd, (void*)&cli_addr, cli_addr_size);
 			if (ret == -1) {
@@ -551,7 +565,7 @@ int connected = 0;
 				return -1;
 			}
 			mslog(s, ctmp, LOG_DEBUG, "passed UDP socket");
-			ctmp->udp_fd_received = 1;
+			ctmp->udp_fd_receive_time = now;
 			connected = 1;
 			
 			reopen_udp_port(listener);
@@ -626,9 +640,8 @@ static int check_tcp_wrapper(int fd)
 int main(int argc, char** argv)
 {
 	int fd, pid, e;
-	struct listener_st *ltmp;
-	struct proc_st *ctmp, *cpos;
-	struct tun_st tun;
+	struct listener_st *ltmp = NULL;
+	struct proc_st *ctmp = NULL, *cpos;
 	fd_set rd;
 	int val, n = 0, ret, flags;
 	struct timeval tv;
@@ -643,9 +656,9 @@ int main(int argc, char** argv)
 	list_head_init(&s.clist.head);
 	list_head_init(&s.ban_list.head);
 	list_head_init(&s.script_list.head);
-	tun_st_init(&tun);
 	tls_cache_init(&s.tls_db);
-
+	ip_lease_init(&s.ip_leases);
+	
 	ocsignal(SIGINT, handle_term);
 	ocsignal(SIGTERM, handle_term);
 	ocsignal(SIGPIPE, SIG_IGN);
@@ -677,7 +690,6 @@ int main(int argc, char** argv)
 	}
 
 	s.config = &config;
-	s.tun = &tun;
 
 	main_auth_init(&s);
 
@@ -690,7 +702,7 @@ int main(int argc, char** argv)
 
 	flags = LOG_PID|LOG_NDELAY;
 #ifdef LOG_PERROR
-	if (config.debug != 0)
+	if (config.debug != 0 || config.http_debug != 0 || config.tls_debug != 0)
 		flags |= LOG_PERROR;
 #endif
 	openlog("ocserv", flags, LOG_DAEMON);
@@ -716,6 +728,7 @@ int main(int argc, char** argv)
 	/* Initialize certificates */
 	tls_global_init_certs(&s);
 
+	mslog(&s, NULL, LOG_INFO, "initialized %s", PACKAGE_STRING);
 	alarm(MAINTAINANCE_TIME(&s));
 
 	for (;;) {
@@ -811,6 +824,7 @@ int main(int argc, char** argv)
 
 					setproctitle(PACKAGE_NAME"-worker");
 					kill_on_parent_kill(SIGTERM);
+					
 
 					ws.config = &config;
 					ws.cmd_fd = cmd_fd[1];
@@ -842,6 +856,9 @@ fork_failed:
 					set_cloexec_flag (cmd_fd[0], 1);
 
 					list_add(&s.clist.head, &(ctmp->list));
+
+					put_into_cgroup(&s, s.config->cgroup, pid);
+
 					s.active_clients++;
 				}
 				close(cmd_fd[1]);
@@ -867,7 +884,7 @@ fork_failed:
 				ret = handle_commands(&s, ctmp);
 				if (ret < 0) {
 					remove_from_script_list(&s, ctmp);
-					remove_proc(&s, ctmp, (ret==ERR_BAD_COMMAND)?1:0);
+					remove_proc(&s, ctmp, (ret!=ERR_WORKER_TERMINATED)?1:0);
 				}
 			}
 		}
