@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
+ *
+ * Author: Nikos Mavrogiannopoulos
+ *
+ * This file is part of ocserv.
+ *
+ * The GnuTLS is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ */
 #ifndef MAIN_H
 # define MAIN_H
 
@@ -48,14 +68,18 @@ enum {
 	PS_AUTH_COMPLETED,
 };
 
+/* Each worker process maps to a unique proc_st structure.
+ */
 struct proc_st {
 	struct list_node list;
-	int fd;
+	int fd; /* the command file descriptor */
 	pid_t pid;
-	unsigned udp_fd_received; /* if the corresponding process has received a UDP fd */
+	time_t udp_fd_receive_time; /* when the corresponding process has received a UDP fd */
 	
 	/* the tun lease this process has */
-	struct lease_st* lease;
+	struct tun_lease_st tun_lease;
+	struct ip_lease_st *ipv4;
+	struct ip_lease_st *ipv6;
 
 	struct sockaddr_storage remote_addr; /* peer address */
 	socklen_t remote_addr_len;
@@ -72,9 +96,22 @@ struct proc_st {
 	char hostname[MAX_HOSTNAME_SIZE]; /* the requested hostname */
 	uint8_t cookie[COOKIE_SIZE]; /* the cookie associated with the session */
 
+	/* if the session is initiated by a cookie the following two are set
+	 * and are considered when generating an IP address.
+	 */
+	uint8_t seeds_are_set; /* non zero if the following two elements are set */
+	uint8_t ipv4_seed[4];
+
 	void * auth_ctx; /* the context of authentication */
 	unsigned auth_status; /* PS_AUTH_ */
 	unsigned auth_reqs; /* the number of requests received */
+	
+	unsigned applied_iroutes; /* whether the iroutes in the config have been successfully applied */
+	struct group_cfg_st config; /* custom user/group config */
+};
+
+struct ip_lease_db_st {
+	struct htable ht;
 };
 
 struct proc_list_st {
@@ -99,7 +136,9 @@ struct ban_list_st {
 
 typedef struct main_server_st {
 	struct cfg_st *config;
-	struct tun_st *tun;
+	
+	struct ip_lease_db_st ip_leases;
+
 	hash_db_st *tls_db;
 	
 	uint8_t cookie_key[16];
@@ -155,8 +194,10 @@ int open_tun(main_server_st* s, struct proc_st* proc);
 int set_tun_mtu(main_server_st* s, struct proc_st * proc, unsigned mtu);
 
 int send_auth_reply_msg(main_server_st* s, struct proc_st* proc);
+
 int send_auth_reply(main_server_st* s, struct proc_st* proc,
 			cmd_auth_reply_t r);
+
 int handle_auth_cookie_req(main_server_st* s, struct proc_st* proc,
  			   const struct cmd_auth_cookie_req_st * req);
 int generate_cookie(main_server_st *s, struct proc_st* proc);
@@ -175,4 +216,11 @@ int handle_script_exit(main_server_st *s, struct proc_st* proc, int code);
 
 void run_sec_mod(main_server_st * s);
 
+int parse_group_cfg_file(main_server_st* s, const char* file, struct group_cfg_st *config);
+
+void del_additional_config(struct group_cfg_st* config);
+void remove_proc(main_server_st* s, struct proc_st *proc, unsigned k);
+
+void put_into_cgroup(main_server_st * s, const char* cgroup, pid_t pid);
+
 #endif
