@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -26,6 +27,16 @@
 
 #ifdef HAVE_PAM
 
+/* A simple PAM authenticator based on coroutines (to achieve
+ * asynchronous operation). It does not use pam_open_session()
+ * as it is unclear to me whether this can have any benefit in our
+ * use cases (and it can actually degrate the server if one sets
+ * limits with it).
+ *
+ * As it is now it only provides authentication via PAM, but
+ * no session management.
+ */
+
 #include <security/pam_appl.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -33,7 +44,7 @@
 #include <pcl.h>
 #include <str.h>
 
-#define PAM_STACK_SIZE (48*1024)
+#define PAM_STACK_SIZE (128*1024)
 
 #define MAX_REPLIES 2
 
@@ -269,6 +280,29 @@ struct pam_ctx_st * pctx = ctx;
 	return 0;
 }
 
+static int pam_auth_user(void* ctx, char *username, int username_size)
+{
+const char* user = NULL;
+struct pam_ctx_st * pctx = ctx;
+int pret;
+
+	username[0] = 0;
+
+	pret = pam_get_item(pctx->ph, PAM_USER, (const void **)&user);
+	if (pret != PAM_SUCCESS) {
+		/*syslog(LOG_AUTH, "Error in pam_get_item(PAM_USER): %s", pam_strerror(pctx->ph, pret));*/
+		return -1;
+	}
+	
+	if (user != NULL) {
+		snprintf(username, username_size, "%s", user);
+
+		return 0;
+	}
+	
+	return -1;
+}
+
 static void pam_auth_deinit(void* ctx)
 {
 struct pam_ctx_st * pctx = ctx;
@@ -287,7 +321,8 @@ const struct auth_mod_st pam_auth_funcs = {
   .auth_deinit = pam_auth_deinit,
   .auth_msg = pam_auth_msg,
   .auth_pass = pam_auth_pass,
-  .auth_group = pam_auth_group
+  .auth_group = pam_auth_group,
+  .auth_user = pam_auth_user
 };
 
 #endif
