@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -162,6 +163,10 @@ gnutls_datum_t data, out;
 uint16_t length;
 struct iovec iov[2];
 int sd;
+#if defined(SO_PEERCRED) && defined(HAVE_STRUCT_UCRED)
+struct ucred cr;
+socklen_t cr_len;
+#endif
 
 	ocsignal(SIGHUP, SIG_IGN);
 	ocsignal(SIGINT, SIG_DFL);
@@ -256,11 +261,26 @@ int sd;
 		cfd = accept(sd, (struct sockaddr *)&sa, &sa_len);
 		if (cfd == -1) {
 		 	e = errno;
-		 	syslog(LOG_ERR, "error accepting sec-mod connection: %s", strerror(e));
+		 	syslog(LOG_ERR, "sec-mod error accepting connection: %s", strerror(e));
 		 	continue;
 		}
-		 
-		 /* read request */
+
+#if defined(SO_PEERCRED) && defined(HAVE_STRUCT_UCRED)
+		cr_len = sizeof(cr);
+		ret = getsockopt(cfd, SOL_SOCKET, SO_PEERCRED, &cr, &cr_len);
+		if (ret == -1) {
+		 	e = errno;
+		 	syslog(LOG_ERR, "sec-mod error obtaining peer credentials: %s", strerror(e));
+		 	goto cont;
+		}
+
+	 	syslog(LOG_DEBUG, "sec-mod received request from pid %u and uid %u", (unsigned)cr.pid, (unsigned)cr.uid);
+		if (cr.uid != config->uid || cr.gid != config->gid) {
+		 	syslog(LOG_ERR, "sec-mod received unauthorized request from pid %u and uid %u", (unsigned)cr.pid, (unsigned)cr.uid);
+		 	goto cont;
+		}
+#endif
+		/* read request */
 		ret = recv(cfd, buffer, buffer_size, 0);
 		if (ret == 0)
 			goto cont;
@@ -308,7 +328,11 @@ int sd;
 		
 		iov[1].iov_base = out.data;
 		iov[1].iov_len = out.size;
-		writev(cfd, iov, 2);
+		ret = writev(cfd, iov, 2);
+		if (ret == -1) {
+		        e = errno;
+		 	syslog(LOG_ERR, "sec-mod error in writev: %s", strerror(e));
+		}
 
 		gnutls_free(out.data);
 cont:
