@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -153,7 +154,7 @@ static size_t rehash(const void *_e, void *unused)
 {
 const tls_cache_st *e = _e;
 
-	return hash_stable_8(e->session_id, e->session_id_size, 0);
+	return hash_any(e->session_id, e->session_id_size, 0);
 }
 
 void tls_cache_init(hash_db_st** _db)
@@ -232,7 +233,7 @@ static int verify_certificate_cb(gnutls_session_t session)
 	 */
 	ret = gnutls_certificate_verify_peers2(session, &status);
 	if (ret < 0) {
-		oclog(ws, LOG_ERR, "error verifying client certificate");
+		oclog(ws, LOG_ERR, "error verifying client certificate: %s", gnutls_strerror(ret));
 		goto fail;
 	}
 
@@ -659,15 +660,12 @@ unsigned i;
 
 size_t tls_get_overhead(gnutls_protocol_t version, gnutls_cipher_algorithm_t cipher, gnutls_mac_algorithm_t mac)
 {
-unsigned iv_size, overhead = 0, t;
-unsigned block_size;
-
-	block_size = gnutls_cipher_get_block_size(cipher);
-#if GNUTLS_VERSION_NUMBER >= 0x030200
-	iv_size = gnutls_cipher_get_iv_size(cipher);
+#if GNUTLS_VERSION_NUMBER >= 0x030207
+	return gnutls_est_record_overhead_size(version, cipher, mac, GNUTLS_COMP_NULL, 0);
 #else
-	iv_size = block_size;
-#endif
+unsigned overhead = 0, t;
+unsigned block_size;
+	block_size = gnutls_cipher_get_block_size(cipher);
 	
 	switch(version) {
 		case GNUTLS_DTLS0_9:
@@ -691,11 +689,11 @@ unsigned block_size;
 		case GNUTLS_CIPHER_AES_192_CBC:
 		case GNUTLS_CIPHER_CAMELLIA_192_CBC:
 			overhead += block_size; /* max pad */
-			overhead += iv_size; /* explicit IV */
+			overhead += block_size; /* IV */
 			break;
 		case GNUTLS_CIPHER_AES_128_GCM:
 		case GNUTLS_CIPHER_AES_256_GCM:
-			overhead += iv_size; /* explicit IV */
+			overhead += 8; /* explicit nonce */
 			overhead += block_size; /* tag size */
 			break;
 		default:
@@ -707,4 +705,5 @@ unsigned block_size;
 		overhead += t;
 		
 	return overhead;
+#endif
 }
