@@ -1,9 +1,29 @@
+/*
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
+ *
+ * Author: Nikos Mavrogiannopoulos
+ *
+ * This file is part of ocserv.
+ *
+ * The GnuTLS is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ */
 #ifndef VPN_H
 #define VPN_H
 
 #include <config.h>
 #include <gnutls/gnutls.h>
-#include <http-parser/http_parser.h>
+#include <http_parser.h>
 #include <ccan/htable/htable.h>
 #include <syslog.h>
 #include <sys/types.h>
@@ -47,6 +67,13 @@ extern int syslog_open;
 #define ERR_AUTH_CONTINUE -4
 #define ERR_WAIT_FOR_SCRIPT -5
 #define ERR_MEM -6
+#define ERR_READ_CONFIG -7
+#define ERR_NO_IP -8
+#define ERR_PARSING -9
+#define ERR_EXEC -10
+#define ERR_WORKER_TERMINATED -11
+
+#define LOG_HTTP_DEBUG 2048
 
 typedef struct 
 {
@@ -54,6 +81,32 @@ typedef struct
 	unsigned int entries;
 } hash_db_st;
 
+struct group_cfg_st {
+	/* routes to be forwarded to the client */
+	char **routes;
+	unsigned int routes_size;
+
+	/* routes to be applied to the server */
+	char **iroutes;
+	unsigned int iroutes_size;
+
+	char *ipv4_dns;
+	char *ipv6_dns;
+	char *ipv4_nbns;
+	char *ipv6_nbns;
+	char *ipv4_network;
+	char *ipv6_network;
+	char *ipv4_netmask;
+	char *ipv6_netmask;
+	
+	char *cgroup;
+
+	size_t rx_per_sec;
+	size_t tx_per_sec;
+	
+	unsigned net_priority;
+};
+
 struct vpn_st {
 	char *name;	/* device name */
 	char *ipv4_netmask;
@@ -104,6 +157,7 @@ struct cfg_st {
 	unsigned dpd;
 	unsigned foreground;
 	unsigned tls_debug;
+	unsigned http_debug;
 	unsigned debug;
 	unsigned max_clients;
 	unsigned max_same_clients;
@@ -113,11 +167,20 @@ struct cfg_st {
 	unsigned rate_limit_ms; /* if non zero force a connection every rate_limit milliseconds */
 	unsigned ping_leases; /* non zero if we need to ping prior to leasing */
 	
+	size_t rx_per_sec;
+	size_t tx_per_sec;
+	unsigned net_priority;
+
 	unsigned output_buffer;
 	unsigned default_mtu;
 
+	char *route_add_cmd;
+	char *route_del_cmd;
+
 	char *connect_script;
 	char *disconnect_script;
+	
+	char *cgroup;
 
 #ifdef ANYCONNECT_CLIENT_COMPAT
 	char *xml_config_file;
@@ -127,6 +190,10 @@ struct cfg_st {
 
 	uid_t uid;
 	gid_t gid;
+
+	/* additional configuration files */
+	char *per_group_dir;
+	char *per_user_dir;
 	
 	/* the tun network */
 	struct vpn_st network;
@@ -144,6 +211,10 @@ struct main_server_st;
 #define MAX_GROUPNAME_SIZE MAX_USERNAME_SIZE
 #define MAX_SESSION_DATA_SIZE (4*1024)
 
+#define MAX_CONFIG_ENTRIES 64
+
+#define MAX_ROUTES 32
+
 #include <tun.h>
 
 const char *human_addr(const struct sockaddr *sa, socklen_t salen,
@@ -160,4 +231,13 @@ const char *human_addr(const struct sockaddr *sa, socklen_t salen,
 
 #define SA_IN_P_GENERIC(addr, size) ((size==sizeof(struct sockaddr_in))?SA_IN_U8_P(addr):SA_IN6_U8_P(addr))
 #define SA_IN_SIZE(size) ((size==sizeof(struct sockaddr_in))?sizeof(struct in_addr):sizeof(struct in6_addr))
+
+/* macros */
+#define TOS_PACK(x) (x<<4)
+#define TOS_UNPACK(x) (x>>4)
+#define IS_TOS(x) ((x&0x0f)==0)
+
+/* Helper structures */
+enum option_types { OPTION_NUMERIC, OPTION_STRING, OPTION_BOOLEAN, OPTION_MULTI_LINE };
+
 #endif
