@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -37,9 +38,10 @@
 #include "html.h"
 #include <worker.h>
 #include <cookies.h>
+#include <common.h>
 #include <tlslib.h>
 
-#include <http-parser/http_parser.h>
+#include <http_parser.h>
 
 #define SUCCESS_MSG_HEAD "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" \
                         "<auth id=\"success\">\n" \
@@ -173,6 +175,7 @@ static int send_auth_req(int fd, const struct cmd_auth_req_st* r)
 	struct iovec iov[2];
 	uint8_t cmd;
 	struct msghdr hdr;
+	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
 	
@@ -187,7 +190,12 @@ static int send_auth_req(int fd, const struct cmd_auth_req_st* r)
 	hdr.msg_iov = iov;
 	hdr.msg_iovlen = 2;
 
-	return(sendmsg(fd, &hdr, 0));
+	ret = sendmsg(fd, &hdr, 0);
+	if (ret < 0) {
+		int e = errno;
+		syslog(LOG_ERR, "send_auth_req: sendmsg: %s", strerror(e));
+	}
+	return ret;
 }
 
 static int send_auth_init(int fd, const struct cmd_auth_init_st* r)
@@ -195,6 +203,7 @@ static int send_auth_init(int fd, const struct cmd_auth_init_st* r)
 	struct iovec iov[2];
 	uint8_t cmd;
 	struct msghdr hdr;
+	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
 	
@@ -209,7 +218,12 @@ static int send_auth_init(int fd, const struct cmd_auth_init_st* r)
 	hdr.msg_iov = iov;
 	hdr.msg_iovlen = 2;
 
-	return(sendmsg(fd, &hdr, 0));
+	ret = sendmsg(fd, &hdr, 0);
+	if (ret < 0) {
+		int e = errno;
+		syslog(LOG_ERR, "send_auth_req: sendmsg: %s", strerror(e));
+	}
+	return ret;
 }
 
 static int send_auth_cookie_req(int fd, const struct cmd_auth_cookie_req_st* r)
@@ -217,6 +231,7 @@ static int send_auth_cookie_req(int fd, const struct cmd_auth_cookie_req_st* r)
 	struct iovec iov[2];
 	uint8_t cmd;
 	struct msghdr hdr;
+	int ret;
 
 	memset(&hdr, 0, sizeof(hdr));
 	
@@ -231,71 +246,249 @@ static int send_auth_cookie_req(int fd, const struct cmd_auth_cookie_req_st* r)
 	hdr.msg_iov = iov;
 	hdr.msg_iovlen = 2;
 
-	return(sendmsg(fd, &hdr, 0));
+	ret = sendmsg(fd, &hdr, 0);
+	if (ret < 0) {
+		int e = errno;
+		syslog(LOG_ERR, "send_auth_req: sendmsg: %s", strerror(e));
+	}
+	return ret;
 }
 
-static int recv_auth_reply(worker_st *ws, struct cmd_auth_reply_st *resp)
+static int recv_value_length(worker_st *ws, str_st* b)
 {
-	struct iovec iov[2];
-	uint8_t cmd = 0;
+int ret;
+uint16_t len;
+
+	ret = force_read(ws->cmd_fd, &len, 2);
+	if (ret != 2) {
+		oclog(ws, LOG_ERR, "Error receiving length-value from main (%d)", ret);
+		return ERR_BAD_COMMAND;
+	}
+				
+	if (len > 0) {
+		ret = str_append_size(b, len);
+		if (ret < 0) {
+			oclog(ws, LOG_ERR, "Memory error in str_append_size()");
+			return ret;
+		}
+		
+		ret = force_read(ws->cmd_fd, b->data, len);
+		if (ret != len) {
+			oclog(ws, LOG_ERR, "Error receiving value from main (%d)", ret);
+			return ERR_BAD_COMMAND;
+		}
+		b->length += len;
+		b->data[len] = 0;
+	}
+	
+	return 0;
+}
+
+static
+int deserialize_additional_config(worker_st* ws)
+{
+int ret;
+unsigned i;
+str_st b;
+uint32_t t;
+char *st;
+
+	str_init(&b);
+	
+	ret = recv_value_length(ws, &b);
+	if (ret < 0)
+		goto cleanup;
+	
+	/* IPV4 DNS */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv4_dns);
+		ws->config->network.ipv4_dns = st;
+	}
+
+	/* IPV6 DNS */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv6_dns);
+		ws->config->network.ipv6_dns = st;
+	}
+
+	/* IPV4 NBNS */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv4_nbns);
+		ws->config->network.ipv4_nbns = st;
+	}
+
+	/* IPV6 NBNS */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv6_nbns);
+		ws->config->network.ipv6_nbns = st;
+	}
+
+	/* IPV4 netmask */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv4_netmask);
+		ws->config->network.ipv4_netmask = st;
+	}
+
+	/* IPV6 netmask */
+	ret = str_read_str_prefix1(&b, &st, NULL);
+	if (ret < 0)
+		goto cleanup;
+	if (st != NULL) {
+		free(ws->config->network.ipv6_netmask);
+		ws->config->network.ipv6_netmask = st;
+	}
+
+	/* number of routes */
+	if (b.length < 1) {
+		oclog(ws, LOG_ERR, "Error in received length-value from main");
+		ret = ERR_BAD_COMMAND;
+		goto cleanup;
+	}
+	
+	ret = str_read_data(&b, &t, sizeof(t));
+	if (ret < 0)
+		goto cleanup;
+	
+	if (t != 0)
+		ws->config->rx_per_sec = t;
+
+	ret = str_read_data(&b, &t, sizeof(t));
+	if (ret < 0)
+		goto cleanup;
+		
+	if (t != 0)
+		ws->config->tx_per_sec = t;
+
+	ret = str_read_data(&b, &t, sizeof(t));
+	if (ret < 0)
+		goto cleanup;
+		
+	if (t != 0)
+		ws->config->net_priority = t;
+
+	ws->routes_size = b.data[0];
+	b.length--;
+	b.data++;
+
+	/* routes */
+	for (i=0;i<ws->routes_size;i++) {
+		ret = str_read_str_prefix1(&b, &ws->routes[i], NULL);
+		if (ret < 0) {
+			oclog(ws, LOG_ERR, "Error receiving private routes from main");
+			ret = ERR_BAD_COMMAND;
+			goto cleanup;
+		}
+	}
+	
+	ret = 0;
+cleanup:
+	str_clear(&b);
+	return ret;
+}
+
+static int recv_auth_reply(worker_st *ws, struct cmd_auth_reply_msg_st* mresp)
+{
+	struct iovec iov[1];
+	uint8_t cmd[2] = {0};
 	struct msghdr hdr;
-	int ret, cmdlen;
+	int ret;
 	union {
 		struct cmsghdr    cm;
 		char              control[CMSG_SPACE(sizeof(int))];
 	} control_un;
 	struct cmsghdr  *cmptr;
+	struct cmd_auth_reply_info_st resp;
 	
-	iov[0].iov_base = &cmd;
-	iov[0].iov_len = 1;
-
-	iov[1].iov_base = resp;
-	iov[1].iov_len = sizeof(*resp);
+	iov[0].iov_base = cmd;
+	iov[0].iov_len = 2;
 
 	memset(&hdr, 0, sizeof(hdr));
+	memset(&control_un, 0, sizeof(control_un));
+
 	hdr.msg_iov = iov;
-	hdr.msg_iovlen = 2;
+	hdr.msg_iovlen = 1;
 
 	hdr.msg_control = control_un.control;
 	hdr.msg_controllen = sizeof(control_un.control);
 
 	ret = recvmsg( ws->cmd_fd, &hdr, 0);
-	
-	cmdlen = ret;
-	
-	if (cmdlen < 2) {
-		oclog(ws, LOG_ERR, "Received incorrect data (%d, expected %d) from main", cmdlen, (int)2);
+	if (ret != 2) {
+		int e = errno;
+		oclog(ws, LOG_ERR, "auth_reply: incorrect data (%d, expected %d) from main: %s", ret, (int)2, strerror(e));
 		return ERR_AUTH_FAIL;
 	}
-	if (cmd != AUTH_REP)
-		return ERR_AUTH_FAIL;
-		
-	cmdlen--;
 
-	switch(resp->reply) {
+	if (cmd[0] != AUTH_REP) {
+		oclog(ws, LOG_ERR, "auth_reply: received unexpected message (%d)", (int)cmd[0]);
+		return ERR_AUTH_FAIL;
+	}
+		
+	switch(cmd[1]) {
 		case REP_AUTH_MSG:
-			return ERR_AUTH_CONTINUE;
-		case REP_AUTH_OK:
-			if (cmdlen < sizeof(*resp)) {
-				oclog(ws, LOG_ERR, "Received incorrect data (%d, expected %d) from main", ret, (int)sizeof(*resp)+1);
+			if (mresp == NULL) {
+				oclog(ws, LOG_ERR, "recv_auth_reply: received unexpected msg");
+				return ERR_AUTH_FAIL;
+			}
+			
+			ret = force_read(ws->cmd_fd, mresp, sizeof(*mresp));
+			if (ret < sizeof(*mresp)) {
+				int e = errno;
+				oclog(ws, LOG_ERR, "recv_auth_reply_msg: read(%d): %s", ret, strerror(e));
 				return ERR_AUTH_FAIL;
 			}
 
+			return ERR_AUTH_CONTINUE;
+		case REP_AUTH_OK:
 			if ( (cmptr = CMSG_FIRSTHDR(&hdr)) != NULL && cmptr->cmsg_len == CMSG_LEN(sizeof(int))) {
-				if (cmptr->cmsg_level != SOL_SOCKET)
-					return -1;
-				if (cmptr->cmsg_type != SCM_RIGHTS)
+
+				if (cmptr->cmsg_level != SOL_SOCKET || cmptr->cmsg_type != SCM_RIGHTS) {
+					oclog(ws, LOG_ERR, "recv_auth_reply: incorrect message type");
 					return ERR_AUTH_FAIL;
-				
+				}
+
 				memcpy(&ws->tun_fd, CMSG_DATA(cmptr), sizeof(int));
-				memcpy(ws->tun_name, resp->vname, sizeof(ws->tun_name));
-				memcpy(ws->username, resp->user, sizeof(ws->username));
-				memcpy(ws->cookie, resp->cookie, sizeof(ws->cookie));
-				memcpy(ws->session_id, resp->session_id, sizeof(ws->session_id));
-			} else
+
+				ret = force_read(ws->cmd_fd, &resp, sizeof(resp));
+				if (ret < sizeof(resp)) {
+					int e = errno;
+					oclog(ws, LOG_ERR, "recv_auth_reply: read(%d): %s", ret, strerror(e));
+					return ERR_AUTH_FAIL;
+				}
+				
+				memcpy(ws->tun_name, resp.vname, sizeof(ws->tun_name));
+				memcpy(ws->username, resp.user, sizeof(ws->username));
+				memcpy(ws->cookie, resp.cookie, sizeof(ws->cookie));
+				memcpy(ws->session_id, resp.session_id, sizeof(ws->session_id));
+
+				/* Read any additional data */
+
+				ret = deserialize_additional_config(ws);
+				if (ret < 0) {
+					oclog(ws, LOG_ERR, "recv_auth_reply: deserialize failed");
+					return ERR_AUTH_FAIL;
+				}
+			} else {
+				oclog(ws, LOG_ERR, "recv_auth_reply: error in received message");
 				return ERR_AUTH_FAIL;
+			}
 			break;
 		default:
+			oclog(ws, LOG_ERR, "recv_auth_reply: unexpected command");
 			return ERR_AUTH_FAIL;
 	}
 	
@@ -354,12 +547,13 @@ int auth_cookie(worker_st *ws, void* cookie, size_t cookie_size)
 {
 int ret;
 struct cmd_auth_cookie_req_st areq;
-struct cmd_auth_reply_st resp;
 
 	memset(&areq, 0, sizeof(areq));
 
-	if (cookie_size != sizeof(areq.cookie))
+	if (cookie_size != sizeof(areq.cookie)) {
+		oclog(ws, LOG_INFO, "unexpected cookie size (%d, expected %d)", (int)cookie_size, (int)sizeof(areq.cookie));
 		return -1;
+	}
 
 	if ((ws->config->auth_types & AUTH_TYPE_CERTIFICATE) && ws->config->force_cert_auth != 0) {
 		if (ws->cert_auth_ok == 0) {
@@ -369,8 +563,10 @@ struct cmd_auth_reply_st resp;
 
 		ret = get_cert_info(ws, areq.cert_user, sizeof(areq.cert_user),
 					areq.cert_group, sizeof(areq.cert_group));
-		if (ret < 0)
+		if (ret < 0) {
+			oclog(ws, LOG_INFO, "cannot obtain certificate info");
 			return -1;
+		}
 
 		areq.tls_auth_ok = 1;
 	}
@@ -379,10 +575,18 @@ struct cmd_auth_reply_st resp;
 
 	oclog(ws, LOG_DEBUG, "sending cookie authentication request");
 	ret = send_auth_cookie_req(ws->cmd_fd, &areq);
-	if (ret < 0)
+	if (ret < 0) {
+		oclog(ws, LOG_INFO, "error sending cookie authentication request");
 		return ret;
+	}
 
-	return recv_auth_reply(ws, &resp);
+	ret = recv_auth_reply(ws, NULL);
+	if (ret < 0) {
+		oclog(ws, LOG_INFO, "error receiving cookie authentication reply");
+		return ret;
+	}
+	
+	return 0;
 }
 
 int post_common_handler(worker_st *ws, unsigned http_ver)
@@ -481,7 +685,7 @@ int read_user_pass(worker_st *ws, char* body, unsigned body_length, char** usern
 	char *p;
 	
 	if (memmem(body, body_length, "<?xml", 5) != 0) {
-		oclog(ws, LOG_DEBUG, "POST body: '%.*s'", body_length, body);
+		oclog(ws, LOG_HTTP_DEBUG, "POST body: '%.*s'", body_length, body);
 
 		if (username != NULL) {
 			/* body should contain <username>test</username><password>test</password> */
@@ -590,7 +794,7 @@ struct http_req_st *req = &ws->req;
 const char* reason = "Authentication failed";
 char * username = NULL;
 char * password = NULL;
-struct cmd_auth_reply_st resp;
+struct cmd_auth_reply_msg_st resp;
 
 	if (ws->auth_state == S_AUTH_INACTIVE) {
 		struct cmd_auth_init_st areq;
