@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -46,7 +47,7 @@ int get_config_handler(worker_st *ws, unsigned http_ver)
 int ret;
 struct stat st;
 
-	oclog(ws, LOG_DEBUG, "requested config: %s", ws->req.url); 
+	oclog(ws, LOG_HTTP_DEBUG, "requested config: %s", ws->req.url); 
 	if (ws->config->xml_config_file == NULL) {
 		oclog(ws, LOG_INFO, "requested config but no config file is set");
 		tls_printf(ws->session, "HTTP/1.%u 404 Not found\r\n", http_ver);
@@ -107,7 +108,7 @@ int ret;
 const char *data;
 int len;
 
-	oclog(ws, LOG_DEBUG, "requested fixed string: %s", ws->req.url); 
+	oclog(ws, LOG_HTTP_DEBUG, "requested fixed string: %s", ws->req.url); 
 	if (!strcmp(ws->req.url, "/2/binaries/update.txt")) {
 		data = VPN_VERSION;
 		len = sizeof(VPN_VERSION)-1;
@@ -157,7 +158,7 @@ int ret;
 const char *data;
 int len;
 
-	oclog(ws, LOG_DEBUG, "requested downloader: %s", ws->req.url); 
+	oclog(ws, LOG_HTTP_DEBUG, "requested downloader: %s", ws->req.url); 
 
 	data = SH_SCRIPT;
 	len = sizeof(SH_SCRIPT)-1;
