@@ -1,19 +1,20 @@
 /*
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -31,11 +32,11 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <limits.h>
-#include <net/if.h>
 #include <netinet/in.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <net/if.h>
 #include <arpa/inet.h>
 #include <signal.h>
 #include <time.h>
@@ -44,8 +45,6 @@
 #include <worker.h>
 #include <tlslib.h>
 
-#include <http-parser/http_parser.h>
-
 /* if local is non zero it returns the local, otherwise the remote */
 static
 int get_ip(struct worker_st* ws, int fd, int family, unsigned int local,
@@ -167,6 +166,16 @@ struct ifreq ifr;
 	else
 		vinfo->ipv6_dns = ws->config->network.ipv6_dns;
 
+	if (ws->config->network.ipv4_nbns && strcmp(ws->config->network.ipv4_nbns, LOCAL) == 0)
+		vinfo->ipv4_nbns = vinfo->ipv4_local;
+	else
+		vinfo->ipv4_nbns = ws->config->network.ipv4_nbns;
+
+	if (ws->config->network.ipv6_nbns && strcmp(ws->config->network.ipv6_nbns, LOCAL) == 0)
+		vinfo->ipv6_nbns = vinfo->ipv6_local;
+	else
+		vinfo->ipv6_nbns = ws->config->network.ipv6_nbns;
+
 	vinfo->routes_size = ws->config->network.routes_size;
 	if (ws->config->network.routes_size > 0)
 		vinfo->routes = ws->config->network.routes;
