@@ -1,20 +1,20 @@
 /*
- * Copyright (C) 2012, 2013 David Woodhouse
  * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This file is part of ocserv.
+ *
+ * ocserv is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GnuTLS is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,7 +32,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <limits.h>
-#include <net/if.h>
 #include <netinet/in.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -40,7 +39,9 @@
 #include <arpa/inet.h>
 #include <system.h>
 #include <time.h>
+#include <gettime.h>
 #include <common.h>
+#include <worker-bandwidth.h>
 
 #include <vpn.h>
 #include "ipc.h"
@@ -48,7 +49,7 @@
 #include <worker.h>
 #include <tlslib.h>
 
-#include <http-parser/http_parser.h>
+#include <http_parser.h>
 
 /* after that time (secs) of inactivity in the UDP part, connection switches to 
  * TCP (if activity occurs there).
@@ -175,6 +176,12 @@ int url_cb(http_parser* parser, const char *at, size_t length)
 #define STR_HDR_ATYPE "X-CSTP-Address-Type"
 #define STR_HDR_HOST "X-CSTP-Hostname"
 
+
+#define CS_ESALSA20 "OC-DTLS1_2-ESALSA20-SHA"
+#define CS_SALSA20 "OC-DTLS1_2-SALSA20-SHA"
+#define CS_AES128_GCM "OC-DTLS1_2-AES128-GCM"
+#define CS_AES256_GCM "OC-DTLS1_2-AES256-GCM"
+
 static void value_check(struct worker_st *ws, struct http_req_st *req)
 {
 unsigned length;
@@ -186,7 +193,7 @@ char * str;
 	if (req->value.length <= 0)
 		return;
 
-	oclog(ws, LOG_DEBUG, "HTTP: %.*s: %.*s", (int)req->header.length, req->header.data,  
+	oclog(ws, LOG_HTTP_DEBUG, "HTTP: %.*s: %.*s", (int)req->header.length, req->header.data,  
 		(int)req->value.length, req->value.data);
 
 	switch (req->next_header) {
@@ -215,18 +222,35 @@ char * str;
 		case HEADER_DTLS_CIPHERSUITE:
 			str = (char*)req->value.data;
 			while ((token = strtok(str, ":")) != NULL) {
-#if GNUTLS_VERSION_NUMBER >= 0x030201
-				if (strcmp(token, "X-ESTREAM-SALSA20-UMAC96") == 0) {
-				        req->selected_ciphersuite = "X-ESTREAM-SALSA20-UMAC96";
-				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS0.9:+COMP-NULL:+ESTREAM-SALSA20-256:+UMAC-96:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION";
+#if GNUTLS_VERSION_NUMBER >= 0x030207
+				if (strcmp(token, CS_ESALSA20) == 0) {
+				        req->selected_ciphersuite = CS_ESALSA20;
+				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS1.2:+COMP-NULL:+ESTREAM-SALSA20-256:+SHA1:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION";
 				        req->gnutls_cipher = GNUTLS_CIPHER_ESTREAM_SALSA20_256;
-				        req->gnutls_mac = GNUTLS_MAC_UMAC_96;
+				        req->gnutls_mac = GNUTLS_MAC_SHA1;
+				        req->gnutls_version = GNUTLS_DTLS1_2;
 				        break;
-				} else if (strcmp(token, "X-SALSA20-UMAC96") == 0) {
-				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS0.9:+COMP-NULL:+SALSA20-256:+UMAC-96:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION";
-				        req->selected_ciphersuite = "X-SALSA20-UMAC96";
+				} else if (strcmp(token, CS_SALSA20) == 0) {
+				        req->selected_ciphersuite = CS_SALSA20;
+				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS1.2:+COMP-NULL:+SALSA20-256:+SHA1:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION";
 				        req->gnutls_cipher = GNUTLS_CIPHER_SALSA20_256;
-				        req->gnutls_mac = GNUTLS_MAC_UMAC_96;
+				        req->gnutls_mac = GNUTLS_MAC_SHA1;
+				        req->gnutls_version = GNUTLS_DTLS1_2;
+				        break;
+                                } else
+	                        if (strcmp(token, CS_AES128_GCM) == 0) {
+				        req->selected_ciphersuite = CS_AES128_GCM;
+				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-128-GCM:+AEAD:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION:+SIGN-ALL";
+				        req->gnutls_cipher = GNUTLS_CIPHER_AES_128_GCM;
+				        req->gnutls_mac = GNUTLS_MAC_AEAD;
+				        req->gnutls_version = GNUTLS_DTLS1_2;
+				        break;
+	                        } else if (strcmp(token, CS_AES256_GCM) == 0) {
+				        req->selected_ciphersuite = CS_AES256_GCM;
+				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS1.2:+COMP-NULL:+AES-256-GCM:+AEAD:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION:+SIGN-ALL";
+				        req->gnutls_cipher = GNUTLS_CIPHER_AES_256_GCM;
+				        req->gnutls_mac = GNUTLS_MAC_AEAD;
+				        req->gnutls_version = GNUTLS_DTLS1_2;
 				        break;
 	                        } else
 #endif
@@ -235,12 +259,14 @@ char * str;
 				        req->selected_ciphersuite = "AES128-SHA";
 				        req->gnutls_cipher = GNUTLS_CIPHER_AES_128_CBC;
 				        req->gnutls_mac = GNUTLS_MAC_SHA1;
+				        req->gnutls_version = GNUTLS_DTLS0_9;
 				        break;
 				} else if (strcmp(token, "DES-CBC3-SHA") == 0) {
 				        req->gnutls_ciphersuite = "NONE:+VERS-DTLS0.9:+COMP-NULL:+3DES-CBC:+SHA1:+RSA:%COMPAT:%DISABLE_SAFE_RENEGOTIATION";
 				        req->selected_ciphersuite = "DES-CBC3-SHA";
 				        req->gnutls_cipher = GNUTLS_CIPHER_3DES_CBC;
 				        req->gnutls_mac = GNUTLS_MAC_SHA1;
+				        req->gnutls_version = GNUTLS_DTLS0_9;
 				        break;
 	                        }
 				str = NULL;
@@ -383,7 +409,7 @@ int body_cb(http_parser* parser, const char *at, size_t length)
 	struct http_req_st *req = &ws->req;
 	char* tmp;
 	
-	tmp = realloc(req->body, req->body_length+length+1);
+	tmp = safe_realloc(req->body, req->body_length+length+1);
 	if (tmp == NULL)
 		return 1;
 		
@@ -424,7 +450,7 @@ gnutls_datum_t sid = { ws->session_id, sizeof(ws->session_id) };
 	}
 
 	ret = gnutls_session_set_premaster(session, GNUTLS_SERVER,
-		GNUTLS_DTLS0_9, GNUTLS_KX_RSA, ws->req.gnutls_cipher,
+		ws->req.gnutls_version, GNUTLS_KX_RSA, ws->req.gnutls_cipher,
 		ws->req.gnutls_mac, GNUTLS_COMP_NULL, &master, &sid);
 	if (ret < 0) {
 		oclog(ws, LOG_ERR, "could not set TLS premaster: %s", gnutls_strerror(ret));
@@ -586,7 +612,7 @@ restart:
 	} while(ws->req.headers_complete == 0);
 
 	if (parser.method == HTTP_GET) {
-		oclog(ws, LOG_DEBUG, "HTTP GET %s", ws->req.url); 
+		oclog(ws, LOG_HTTP_DEBUG, "HTTP GET %s", ws->req.url); 
 		fn = get_url_handler(ws->req.url);
 		if (fn == NULL) {
 			oclog(ws, LOG_INFO, "unexpected URL %s", ws->req.url);
@@ -599,7 +625,7 @@ restart:
 
 	} else if (parser.method == HTTP_POST) {
 		/* continue reading */
-		oclog(ws, LOG_DEBUG, "HTTP POST %s", ws->req.url); 
+		oclog(ws, LOG_HTTP_DEBUG, "HTTP POST %s", ws->req.url); 
 		while(ws->req.message_complete == 0) {
 			nrecvd = tls_recv(session, buf, sizeof(buf));
 			GNUTLS_FATAL_ERR(nrecvd);
@@ -623,7 +649,7 @@ restart:
 			goto restart;
 
 	} else if (parser.method == HTTP_CONNECT) {
-		oclog(ws, LOG_DEBUG, "HTTP CONNECT %s", ws->req.url); 
+		oclog(ws, LOG_HTTP_DEBUG, "HTTP CONNECT %s", ws->req.url); 
 		ret = connect_handler(ws);
 		if (ret == 0 && (parser.http_major != 1 || parser.http_minor != 0))
 			goto restart;
@@ -719,8 +745,16 @@ int max, e, ret;
 	}
 	if (now-ws->last_msg_tcp > DPD_TRIES*ws->config->dpd) {
 		oclog(ws, LOG_ERR, "have not received TCP DPD for long (%d secs)", (int)(now-ws->last_msg_tcp));
-		ws->buffer[0] = AC_PKT_DPD_OUT;
-		tls_send(ws->session, ws->buffer, 1);
+		ws->buffer[0] = 'S';
+		ws->buffer[1] = 'T';
+		ws->buffer[2] = 'F';
+		ws->buffer[3] = 1;
+		ws->buffer[4] = 0;
+		ws->buffer[5] = 0;
+		ws->buffer[6] = AC_PKT_DPD_OUT;
+		ws->buffer[7] = 0;
+
+		tls_send(ws->session, ws->buffer, 8);
 
 		if (now-ws->last_msg_tcp > DPD_MAX_TRIES*ws->config->dpd) {
 			oclog(ws, LOG_ERR, "have not received TCP DPD for very long; tearing down connection");
@@ -748,6 +782,36 @@ int max, e, ret;
 	return 0;
 }
 
+#define TOSCLASS(x) (IPTOS_CLASS_CS##x)
+
+static void set_net_priority(worker_st *ws, int fd, int priority)
+{
+int t;
+int ret;
+#if defined(IP_TOS)
+	if (priority != 0 && IS_TOS(priority)) {
+		t = TOS_UNPACK(priority);
+		ret = setsockopt( fd, IPPROTO_IP, IP_TOS, &t, sizeof(t));
+		if (ret == -1)
+			oclog(ws, LOG_DEBUG, "setsockopt(IP_TOS) to %x, failed.", (unsigned)t);
+
+		return;
+	}
+#endif
+
+#ifdef SO_PRIORITY
+	if (priority != 0 && priority <= 7) {
+		t = ws->config->net_priority - 1;
+		ret = setsockopt( fd, SOL_SOCKET, SO_PRIORITY, &t, sizeof(t));
+		if (ret == -1)
+			oclog(ws, LOG_DEBUG, "setsockopt(SO_PRIORITY) to %d, failed.", t);
+		
+		return;
+	}
+#endif
+	return;
+}
+
 #define CSTP_DTLS_OVERHEAD 1
 #define CSTP_OVERHEAD 8
 
@@ -763,8 +827,12 @@ char *p;
 struct timeval tv;
 unsigned tls_pending, dtls_pending = 0, i;
 time_t udp_recv_time = 0, now;
+struct timespec tnow;
 unsigned mtu_overhead = 0;
+int sndbuf;
 socklen_t sl;
+bandwidth_st b_tx;
+bandwidth_st b_rx;
 
 	ws->buffer_size = 16*1024;
 	ws->buffer = malloc(ws->buffer_size);
@@ -837,7 +905,7 @@ socklen_t sl;
         }
 
 	ws->udp_state = UP_DISABLED;
-	if (req->master_secret_set != 0) {
+	if (ws->config->udp_port != 0 && req->master_secret_set != 0) {
 		memcpy(ws->master_secret, req->master_secret, TLS_MASTER_SIZE);
 		ws->udp_state = UP_WAIT_FD;
 	} else {
@@ -853,10 +921,12 @@ socklen_t sl;
 			ret = tls_printf(ws->session, "X-CSTP-Netmask: %s\r\n", vinfo.ipv4_netmask);
 			SEND_ERR(ret);
 		}
+
 		if (vinfo.ipv4_dns) {
 			ret = tls_printf(ws->session, "X-CSTP-DNS: %s\r\n", vinfo.ipv4_dns);
 			SEND_ERR(ret);
 		}
+
 		if (vinfo.ipv4_nbns) {
 			ret = tls_printf(ws->session, "X-CSTP-NBNS: %s\r\n", vinfo.ipv4_nbns);
 			SEND_ERR(ret);
@@ -872,10 +942,12 @@ socklen_t sl;
 			ret = tls_printf(ws->session, "X-CSTP-Netmask: %s\r\n", vinfo.ipv6_netmask);
 			SEND_ERR(ret);
 		}
+		
 		if (vinfo.ipv6_dns) {
 			ret = tls_printf(ws->session, "X-CSTP-DNS: %s\r\n", vinfo.ipv6_dns);
 			SEND_ERR(ret);
 		}
+
 		if (vinfo.ipv6_nbns) {
 			ret = tls_printf(ws->session, "X-CSTP-NBNS: %s\r\n", vinfo.ipv6_nbns);
 			SEND_ERR(ret);
@@ -892,6 +964,17 @@ socklen_t sl;
 			"X-CSTP-Split-Include: %s\r\n", vinfo.routes[i]);
 		SEND_ERR(ret);
 	}
+
+	for (i=0;i<ws->routes_size;i++) {
+		if (req->no_ipv6 != 0 && strchr(ws->routes[i], ':') != 0)
+			continue;
+		if (req->no_ipv4 != 0 && strchr(ws->routes[i], '.') != 0)
+			continue;
+		oclog(ws, LOG_DEBUG, "adding private route %s", ws->routes[i]);
+		ret = tls_printf(ws->session,
+			"X-CSTP-Split-Include: %s\r\n", ws->routes[i]);
+		SEND_ERR(ret);
+	}
 	ret = tls_printf(ws->session, "X-CSTP-Keepalive: %u\r\n", ws->config->keepalive);
 	SEND_ERR(ret);
 
@@ -909,6 +992,7 @@ socklen_t sl;
 		"X-CSTP-Keep: true\r\n"
 		"X-CSTP-TCP-Keepalive: true\r\n"
 		"X-CSTP-Tunnel-All-DNS: false\r\n"
+		"X-CSTP-License: accept\r\n"
 		);
 	SEND_ERR(ret);
 
@@ -937,8 +1021,17 @@ socklen_t sl;
 		ws->conn_mtu = MIN(ws->conn_mtu, max-mtu_overhead);
 	}
 
+	/* set TCP socket options */
+	if (ws->config->output_buffer > 0) {
+		sndbuf = ws->conn_mtu * ws->config->output_buffer;
+		ret = setsockopt( ws->conn_fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, sizeof(sndbuf));
+		if (ret == -1)
+			oclog(ws, LOG_DEBUG, "setsockopt(TCP, SO_SNDBUF) to %u, failed.", sndbuf);
+	}
+
+	set_net_priority(ws, ws->conn_fd, ws->config->net_priority);
+
 	if (ws->udp_state != UP_DISABLED) {
-		int sndbuf;
 
 		p = (char*)ws->buffer;
 		for (i=0;i<sizeof(ws->session_id);i++) {
@@ -977,23 +1070,27 @@ socklen_t sl;
 			oclog(ws, LOG_INFO, "reducing DTLS MTU to peer's DTLS MTU (%u)", req->dtls_mtu);
 		}
 
-		overhead = CSTP_DTLS_OVERHEAD + tls_get_overhead(GNUTLS_DTLS0_9, ws->req.gnutls_cipher, ws->req.gnutls_mac);
+		overhead = CSTP_DTLS_OVERHEAD + tls_get_overhead(ws->req.gnutls_version, ws->req.gnutls_cipher, ws->req.gnutls_mac);
 		dtls_mtu = ws->conn_mtu - overhead;
 
 		tls_printf(ws->session, "X-DTLS-MTU: %u\r\n", dtls_mtu);
-		oclog(ws, LOG_INFO, "suggesting DTLS MTU %u", dtls_mtu);
+		oclog(ws, LOG_DEBUG, "suggesting DTLS MTU %u", dtls_mtu);
 		
 		if (ws->config->output_buffer > 0) {
 			sndbuf = ws->conn_mtu * ws->config->output_buffer;
 			setsockopt( ws->udp_fd, SOL_SOCKET, SO_SNDBUF, &sndbuf, sizeof(sndbuf));
+			if (ret == -1)
+				oclog(ws, LOG_DEBUG, "setsockopt(UDP, SO_SNDBUF) to %u, failed.", sndbuf);
 		}
+
+		set_net_priority(ws, ws->udp_fd, ws->config->net_priority);
 	} else
 		dtls_mtu = 0;
 	
 	if (ws->buffer_size <= ws->conn_mtu+mtu_overhead) {
 		oclog(ws, LOG_WARNING, "buffer size is smaller than MTU (%u < %u); adjusting", ws->buffer_size, ws->conn_mtu);
 		ws->buffer_size = ws->conn_mtu+mtu_overhead;
-		ws->buffer = realloc(ws->buffer, ws->buffer_size);
+		ws->buffer = safe_realloc(ws->buffer, ws->buffer_size);
 		if (ws->buffer == NULL)
 			goto exit;
 	}
@@ -1005,9 +1102,9 @@ socklen_t sl;
 
 	ret = tls_printf(ws->session, "X-CSTP-MTU: %u\r\n", cstp_mtu);
 	SEND_ERR(ret);
-	oclog(ws, LOG_INFO, "suggesting CSTP MTU %u", cstp_mtu);
+	oclog(ws, LOG_DEBUG, "suggesting CSTP MTU %u", cstp_mtu);
 
-	oclog(ws, LOG_INFO, "plaintext MTU is %u", ws->conn_mtu);
+	oclog(ws, LOG_DEBUG, "plaintext MTU is %u", ws->conn_mtu-1);
 	send_tun_mtu(ws, ws->conn_mtu);
 
 	if (ws->config->banner) {
@@ -1022,7 +1119,11 @@ socklen_t sl;
 	SEND_ERR(ret);
 
 	/* start dead peer detection */
-	ws->last_msg_tcp = ws->last_msg_udp = time(0);
+	gettime(&tnow);
+	ws->last_msg_tcp = ws->last_msg_udp = tnow.tv_sec;
+	
+	bandwidth_init(&b_rx, ws->config->rx_per_sec);
+	bandwidth_init(&b_tx, ws->config->tx_per_sec);
 
 	/* main loop  */
 	for(;;) {
@@ -1070,7 +1171,9 @@ socklen_t sl;
 				goto exit;
 			}
 		}
-		now = time(0);
+		gettime(&tnow);
+		now = tnow.tv_sec;
+
 		if (periodic_check(ws, mtu_overhead, now) < 0)
 			goto exit;
 		
@@ -1090,39 +1193,41 @@ socklen_t sl;
 				oclog(ws, LOG_INFO, "TUN device returned zero");
 				continue;
 			}
+			
+			/* only transmit if allowed */
+			if (bandwidth_update(&b_tx, l-1, ws->conn_mtu, &tnow) != 0) {
+				tls_retry = 0;
+				oclog(ws, LOG_DEBUG, "sending %d byte(s)\n", l);
+				if (ws->udp_state == UP_ACTIVE) {
+					ws->buffer[7] = AC_PKT_DATA;
 
-			tls_retry = 0;
-			oclog(ws, LOG_DEBUG, "sending %d byte(s)\n", l);
-			if (ws->udp_state == UP_ACTIVE) {
-				ws->buffer[7] = AC_PKT_DATA;
+					ret = tls_send_nowait(ws->dtls_session, ws->buffer + 7, l + 1);
+					GNUTLS_FATAL_ERR(ret);
 
-				ret = tls_send_nowait(ws->dtls_session, ws->buffer + 7, l + 1);
-				GNUTLS_FATAL_ERR(ret);
+					if (ret == GNUTLS_E_LARGE_PACKET) {
+						mtu_not_ok(ws);
 
-				if (ret == GNUTLS_E_LARGE_PACKET) {
-					mtu_not_ok(ws);
+						oclog(ws, LOG_DEBUG, "retrying (TLS) %d\n", l);
+						tls_retry = 1;
+					} else if (ret >= ws->conn_mtu && ws->config->try_mtu != 0) {
+						mtu_ok(ws);
+					}
+				}
 
-					oclog(ws, LOG_DEBUG, "retrying (TLS) %d\n", l);
-					tls_retry = 1;
-				} else if (ret >= ws->conn_mtu && ws->config->try_mtu != 0) {
-					mtu_ok(ws);
+				if (ws->udp_state != UP_ACTIVE || tls_retry != 0) {
+					ws->buffer[0] = 'S';
+					ws->buffer[1] = 'T';
+					ws->buffer[2] = 'F';
+					ws->buffer[3] = 1;
+					ws->buffer[4] = l >> 8;
+					ws->buffer[5] = l & 0xff;
+					ws->buffer[6] = AC_PKT_DATA;
+					ws->buffer[7] = 0;
+
+					ret = tls_send(ws->session, ws->buffer, l + 8);
+					GNUTLS_FATAL_ERR(ret);
 				}
 			}
-
-			if (ws->udp_state != UP_ACTIVE || tls_retry != 0) {
-				ws->buffer[0] = 'S';
-				ws->buffer[1] = 'T';
-				ws->buffer[2] = 'F';
-				ws->buffer[3] = 1;
-				ws->buffer[4] = l >> 8;
-				ws->buffer[5] = l & 0xff;
-				ws->buffer[6] = AC_PKT_DATA;
-				ws->buffer[7] = 0;
-
-				ret = tls_send(ws->session, ws->buffer, l + 8);
-				GNUTLS_FATAL_ERR(ret);
-			}
-
 		}
 
 		if (FD_ISSET(ws->conn_fd, &rfds) || tls_pending != 0) {
@@ -1135,20 +1240,23 @@ socklen_t sl;
 				oclog(ws, LOG_INFO, "client disconnected");
 				goto exit_nomsg;
 			}
-			
+
+
 			if (ret > 0) {
 				l = ret;
 
-				ret = parse_cstp_data(ws, ws->buffer, l, now);
-				if (ret < 0) {
-					oclog(ws, LOG_INFO, "error parsing CSTP data");
-					goto exit;
-				}
+				if (bandwidth_update(&b_rx, l-8, ws->conn_mtu, &tnow) != 0) {
+					ret = parse_cstp_data(ws, ws->buffer, l, now);
+					if (ret < 0) {
+						oclog(ws, LOG_INFO, "error parsing CSTP data");
+						goto exit;
+					}
 
-				if (ret == AC_PKT_DATA && ws->udp_state == UP_ACTIVE) { 
-					/* client switched to TLS for some reason */
-					if (time(0) - udp_recv_time > UDP_SWITCH_TIME)
-						ws->udp_state = UP_INACTIVE;
+					if (ret == AC_PKT_DATA && ws->udp_state == UP_ACTIVE) { 
+						/* client switched to TLS for some reason */
+						if (now - udp_recv_time > UDP_SWITCH_TIME)
+							ws->udp_state = UP_INACTIVE;
+					}
 				}
 			}
 		}
@@ -1167,16 +1275,18 @@ socklen_t sl;
 						l = ret;
 						ws->udp_state = UP_ACTIVE;
 
-						ret = parse_dtls_data(ws, ws->buffer, l, now);
-						if (ret < 0) {
-							oclog(ws, LOG_INFO, "error parsing CSTP data");
-							goto exit;
+						if (bandwidth_update(&b_rx, l-1, ws->conn_mtu, &tnow) != 0) {
+							ret = parse_dtls_data(ws, ws->buffer, l, now);
+							if (ret < 0) {
+								oclog(ws, LOG_INFO, "error parsing CSTP data");
+								goto exit;
+							}
 						}
 					
 					} else
 						oclog(ws, LOG_DEBUG, "no data received (%d)", ret);
 
-					udp_recv_time = time(0);
+					udp_recv_time = now;
 					break;
 				case UP_SETUP:
 					ret = setup_dtls_connection(ws);
@@ -1209,10 +1319,16 @@ hsk_restart:
 
 					if (ret == 0) {
 						unsigned mtu = gnutls_dtls_get_data_mtu(ws->dtls_session);
+
+						/* openconnect doesn't like if we send more bytes
+						 * than the initially agreed MTU */
+						if (mtu > dtls_mtu) 
+							mtu = dtls_mtu;
+
 						ws->udp_state = UP_ACTIVE;
 						mtu_discovery_init(ws, mtu);
 						mtu_set(ws, mtu);
-						oclog(ws, LOG_INFO, "DTLS handshake completed (MTU: %u)\n", ws->conn_mtu);
+						oclog(ws, LOG_INFO, "DTLS handshake completed (plaintext MTU: %u)\n", ws->conn_mtu-1);
 					}
 					
 					break;
@@ -1315,19 +1431,19 @@ static int parse_cstp_data(struct worker_st* ws,
 int pktlen, ret;
 
 	if (buf_size < 8) {
-		oclog(ws, LOG_INFO, "can't read CSTP header (only %d bytes are available)\n", (int)buf_size);
+		oclog(ws, LOG_INFO, "can't read CSTP header (only %d bytes are available)", (int)buf_size);
 		return -1;
 	}
 
 	if (buf[0] != 'S' || buf[1] != 'T' ||
 	    buf[2] != 'F' || buf[3] != 1 || buf[7]) {
-		oclog(ws, LOG_INFO, "can't recognise CSTP header\n");
+		oclog(ws, LOG_INFO, "can't recognise CSTP header");
 		return -1;
 	}
 
 	pktlen = (buf[4] << 8) + buf[5];
 	if (buf_size != 8 + pktlen) {
-		oclog(ws, LOG_INFO, "unexpected CSTP length\n");
+		oclog(ws, LOG_INFO, "unexpected CSTP length");
 		return -1;
 	}
 
@@ -1345,7 +1461,7 @@ static int parse_dtls_data(struct worker_st* ws,
 int ret;
 
 	if (buf_size < 1) {
-		oclog(ws, LOG_INFO, "can't read DTLS header (only %d bytes are available)\n", (int)buf_size);
+		oclog(ws, LOG_INFO, "can't read DTLS header (only %d bytes are available)", (int)buf_size);
 		return -1;
 	}
 
