@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
+ *
+ * Author: Nikos Mavrogiannopoulos
+ *
+ * This file is part of ocserv.
+ *
+ * The GnuTLS is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ */
 #ifndef WORKER_H
 #define WORKER_H
 
@@ -65,6 +85,7 @@ struct http_req_st {
 	char *selected_ciphersuite; /* static string */
 	int gnutls_cipher;
 	int gnutls_mac;
+	int gnutls_version;
 
 	unsigned int headers_complete;
 	unsigned int message_complete;
@@ -118,6 +139,10 @@ typedef struct worker_st {
 	unsigned cert_auth_ok;
 	int tun_fd;
 	
+	/* additional data - received per user or per group */
+	unsigned routes_size;
+	char* routes[MAX_ROUTES];
+	
 	struct http_req_st req;
 } worker_st;
 
@@ -142,7 +167,6 @@ void __attribute__ ((format(printf, 3, 4)))
 
 int get_rt_vpn_info(worker_st * ws,
                     struct vpn_st* vinfo, char* buffer, size_t buffer_size);
-ssize_t tun_write(int sockfd, const void *buf, size_t len);
 
 int send_tun_mtu(worker_st *ws, unsigned int mtu);
 int handle_worker_commands(struct worker_st *ws);
